package ch.psi.daq.rest.queries;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.query.Aggregation;
import ch.psi.daq.hazelcast.query.AggregationType;
import ch.psi.daq.hazelcast.query.range.QueryRange;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

public class TimeRangeQuery extends AbstractQuery {


   public static final String DATE_FORMAT_STRING = "yyyy/MM/dd hh:mm:ss.SSS";
   private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat(DATE_FORMAT_STRING);
   private static Logger logger = LoggerFactory.getLogger(TimeRangeQuery.class);

   /**
    * 
    * @param ordering whether to add a 'orderBy' clause into the database query
    * @param channelIds all the sourceIds (channel names) we want to query
    * @param fields the fields (who map to fields in the DB) we are interested in returning to the
    *        client, needs to be in insertion order (hence the {@link LinkedHashSet} type)
    * @param binningStrategyEnum
    * @param binDurationOrBinCount
    * @param aggregateChannels
    * @param aggregationType
    * @param aggregations
    * @param queryRange
    * @param startDateTime
    * @param endDateTime
    */
   @JsonCreator
   public TimeRangeQuery(
         // note that those annotations are needed for the polymorphic
         // mapping to work correctly
         @JsonProperty(value = "ordering") Ordering ordering,
         @JsonProperty(value = "channels") List<String> channels,
         @JsonProperty(value = "fields") LinkedHashSet<String> fields,
         @JsonProperty(value = "binningStrategy") BinningStrategyEnum binningStrategyEnum,
         @JsonProperty(value = "binDuration") long binDurationOrBinCount,
         @JsonProperty(value = "aggregateChannels") boolean aggregateChannels,
         @JsonProperty(value = "aggregationType") AggregationType aggregationType,
         @JsonProperty(value = "aggregations") List<Aggregation> aggregations,
         @JsonProperty(value = "queryRange") QueryRange queryRange,
         @JsonProperty(value = "startDateTime") String startDateTime,
         @JsonProperty(value = "endDateTime") String endDateTime) {

      super(ordering, channels, fields, binningStrategyEnum, binDurationOrBinCount, aggregateChannels, aggregationType, aggregations, queryRange);
      
      if (startDateTime != null && endDateTime != null) {
         logger.info("startDateTime and endDateTime specified. This takes precedence over the start / end fields.");
         try {
            Date startDate = DATE_FORMAT.parse(startDateTime);
            Date endDate = DATE_FORMAT.parse(endDateTime);

            getQueryRange().setTimeRange(startDate.getTime(), queryRange.getStartNanos(), endDate.getTime(), queryRange.getEndNanos());
         } catch (ParseException e) {
            logger.error("Parsing the start- and/or endDate was unsuccessful. "
                  + "The format must be '" + DATE_FORMAT_STRING + "'", e);
         }
      }

   }

   /**
    * {@inheritDoc}
    */
   @Override
   public String toString() {
      return ToStringBuilder.reflectionToString(this);
   }

}
